/* conffile.h
 *
 * Generic Config Tools
 *
 * Copyright (C) 2000-2003 Christopher Collins <ccollins@ug.cs.usyd.edu.au>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *                                                                             
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *                                                                             
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
*/

#include <stdio.h>

#ifndef _CONFFILE_H
#define _CONFFILE_H

typedef enum config_type_e {
  ct_boolean,
  ct_integer,
  ct_float,
  ct_string
} config_type_t;

typedef union config_value_u {
  unsigned char         bool;
  long                  integer;
  char *                string;
  float			fp;
} config_value_t;

typedef struct config_s {
	char*                 key;
	config_value_t        value;
	config_type_t         type;
	
	struct config_s*      left;
	struct config_s*      right;
} config_t;

extern int        config_parsevalue(char* strin, config_type_t type, config_value_t* cfgval);
extern int        config_set(config_t* root, char* key, char* value);
extern config_t*  config_bind(config_t* root, char* key, char* value, config_type_t type);
extern int      	config_read(config_t* root, char* filename);
extern config_t*	config_find(config_t* root, char* key);
extern void     	config_free(config_t* root);
extern void		config_print(config_t *root, FILE *fout);

#endif /* _CONFFILE_H */
