/* xfpd.c
 *
 * Chris' FPD
 *
 * Graphics display of FAH protein for Unix
 *
 * Copyright (C) 2003 Christopher Collins <ccollins@ug.cs.usyd.edu.au>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *                                                                             
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *                                                                             
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
*/

/* WHY?
 *
 * Why rework part of FPD?
 *
 * We wanted it for a demo.  OpenGL is good.  GLUT is a nice easy way
 * to do GL with UI, rather than lots of crappy X calls.
 *
 * fpd is also unnecessarily ugly.
*/

#include "xfpd.h"

/* glut objects */
int	mainWin;
int	mainMenu;
int	displaymodeMenu;
int	hideMenu;

/* global configuration */
config_t *	xfpdconf = NULL;

/* quality value to use whilst moving */
static int	staticquality = 0;
static int	movequality = -10;
static int	fastwireframe = 0;

static int		winsize[2];
/* rotation/movement globals */
static GLfloat		lastqrot[4] = { 0.0, 0.0, 0.0, 0.0 };
static int		lpos[2];
static int		spinning = 0;
static int		savemode = 0;

#define		CB_LEFT		1
#define		CB_MIDDLE	2
#define		CB_RIGHT	4
static int		cbuttons = 0; /* we start with all buttons up */

#define		MOVE_NONE	0
#define		MOVE_TRACKBALL	1
#define		MOVE_ZOOM	2
#define		MOVE_INTEREYEADJUST	3
#define		MOVE_DEPTHADJUST	4
int		movemode = MOVE_NONE;

void
loadConfiguration()
{
	config_t *	n;
#ifndef WIN32
#define CFGPATH_SIZE 256
	char cfgpath[CFGPATH_SIZE];
#endif

 	xfpdconf = config_bind(xfpdconf, "stereo_seperation", stringize(INITIAL_INTEREYE),
		ct_float);
	xfpdconf = config_bind(xfpdconf, "stereo_depth", stringize(INITIAL_DEPTHPOINT),
		ct_float);
	xfpdconf = config_bind(xfpdconf, "default_datafile", DEFAULTXYZNAME, 
		ct_string);

	xfpdconf = config_bind(xfpdconf, "quality_static", "0",
		ct_integer);
	xfpdconf = config_bind(xfpdconf, "quality_moving", "-10",
		ct_integer);

	xfpdconf = config_bind(xfpdconf, "background_red", "0.1",
		ct_float);
	xfpdconf = config_bind(xfpdconf, "background_blue", "0.1",
		ct_float);
	xfpdconf = config_bind(xfpdconf, "background_green", "0.1",
		ct_float);

	config_read(xfpdconf, CONFIGFILE);

#ifndef WIN32
	snprintf(cfgpath, CFGPATH_SIZE, "%s/%s", getenv("HOME"), PERSONALCONFIGFILE);
	cfgpath[CFGPATH_SIZE-1]='\0'; /* assert sanity */
	config_read(xfpdconf, cfgpath);
#endif

	/* update built-ins */
	n = config_find(xfpdconf, "stereo_seperation");
	intereye = n->value.fp;

	n = config_find(xfpdconf, "stereo_depth");
	depthpoint = n->value.fp;

	n = config_find(xfpdconf, "default_datafile");
	defaultdatafile = n->value.string;

	n = config_find(xfpdconf, "quality_static");
	staticquality = n->value.integer;

	n = config_find(xfpdconf, "quality_moving");
	movequality = n->value.integer;

	n = config_find(xfpdconf, "background_red");
	bgcolor[0] = n->value.fp;
	n = config_find(xfpdconf, "background_green");
	bgcolor[1] = n->value.fp;
	n = config_find(xfpdconf, "background_blue");
	bgcolor[2] = n->value.fp;
}

void
saveConfiguration()
{
	FILE *		configout = NULL;
	config_t *	n;
#ifndef WIN32
#  define CFGPATH_SIZE 256
	char cfgpath[CFGPATH_SIZE];
#endif

/* update the tree contents */
	n = config_find(xfpdconf, "stereo_seperation");
	n->value.fp = intereye;

	n = config_find(xfpdconf, "stereo_depth");
	n->value.fp = depthpoint;

#ifdef WIN32
	configout = fopen(CONFIGFILE, "wt");
#else 
	snprintf(cfgpath, CFGPATH_SIZE, "%s/%s", getenv("HOME"), PERSONALCONFIGFILE);
	cfgpath[CFGPATH_SIZE-1]='\0'; /* assert sanity */
	configout = fopen(cfgpath, "wt");
#endif
	if (NULL == configout) {
		/* couldn't save config */
		return;
	}
	config_print(xfpdconf, configout);
	fclose(configout);
}

void
usage(char *argv0)
{
	fprintf (stderr, "%s: [<options>]\n", argv0);
	fprintf (stderr, "\n");
	fprintf (stderr, _("Valid Options Are:\n"));
	fprintf (stderr, _("  -nodbuf          : disable double buffering\n"));
	fprintf (stderr, _("  -stereo          : enable stereo\n"));
	fprintf (stderr, _("  -fullscreen      : use fullscreen mode\n"));
	fprintf (stderr, _("  -file <filename> : set xyz data file name\n"));
	fprintf (stderr, _("  -fwf             : use fast wireframe rendering for interactive rotations\n"));
	fprintf (stderr, _("  -h               : this usage message\n"));
}

/* exit function - save preferences and quit */
void
terminateApp(int exitstatus)
{
	saveConfiguration();
	exit(1);
}

/* GLUT display callback */
void
displayFunc(void)
{
	renderMolecule();
	glutSwapBuffers();
}

/* GLUT main window resize callback */
void
resizeFunc(int x, int y)
{
	aspect = (float) x / (float) y;
	glViewport (0, 0, x, y);
	winsize[V_X] = x;
	winsize[V_Y] = y;
}

void
spinIdle(void)
{
	add_quats(lastqrot, qrot, qrot);
	glutPostRedisplay();
}

void
hideMenuFunc(int i)
{
	switch (i) {
	case HM_HYDROGEN:
		oflags ^= OF_IH;
		break;
	case HM_WATER:
		oflags ^= OF_IW;
		break;
	}
	glutPostRedisplay();
}

/* GLUT main menu callback */
void
mainMenuFunc(int i)
{
	switch (i) {
	case MM_SPIN:
		spinning = !spinning;
		if (spinning) {
			glutIdleFunc(spinIdle);
		} else {
			glutIdleFunc(NULL);
		}
		break;
	case MM_STEREOTOGGLE:
		oflags ^= OF_STEREO;
		break;
	case MM_RELOAD:
		if (readxyz() != FAH_OK) {
			fprintf (stderr, _("Failed to read data\n"));
			terminateApp(1);
		}
		glutPostRedisplay();
		break;
	case MM_QUIT:
		/* forcefully exit */
		terminateApp(0);
	default:
		break;
	}
}

/* GLUT display mode menu callback */
void
displaymodeMenuFunc(int i)
{
	/* set the rendering mode flag */
	rendermode = i;

	/* force a redraw */
	glutPostRedisplay();
}

void
mouseButton(int button, int state, int x, int y)
{
	int	maskbit = 0;
	int	modkeys;

	modkeys = glutGetModifiers();

	switch (button) {
	case GLUT_LEFT_BUTTON:
		maskbit = CB_LEFT;
		break;
	case GLUT_MIDDLE_BUTTON:
		maskbit = CB_MIDDLE;
		break;
	case GLUT_RIGHT_BUTTON:
		maskbit = CB_RIGHT;
		break;
	default:
		maskbit = 0;
		break;
	}
	if (state == GLUT_DOWN) {
		cbuttons |= maskbit;
	} else if (state == GLUT_UP) {
		cbuttons &= ~maskbit;
	}
	switch (cbuttons) {
	case CB_LEFT:
		/* support shift-left drag for people stuck with 2button mice */
		if ((modkeys & (GLUT_ACTIVE_SHIFT|GLUT_ACTIVE_CTRL)) == (GLUT_ACTIVE_SHIFT|GLUT_ACTIVE_CTRL)) {
			movemode = MOVE_INTEREYEADJUST;
		} else if (modkeys & GLUT_ACTIVE_SHIFT) {
			movemode = MOVE_ZOOM;
		} else if (modkeys & GLUT_ACTIVE_CTRL) {
			movemode = MOVE_DEPTHADJUST;
		} else {
			movemode = MOVE_TRACKBALL;
		}
		if (movemode != MOVE_NONE) {
			quality = movequality;
			if (fastwireframe 
				&& (rendermode != DM_FWF && rendermode != DM_FAT)) {
				savemode = rendermode;
				if (rendermode == DM_AT) {
					rendermode = DM_FAT;
				} else {
					rendermode = DM_FWF;
				}
			}
		}
		lpos[V_X] = x;
		lpos[V_Y] = y;
		glutPostRedisplay();
		break;
	case CB_MIDDLE:
		quality = movequality;
		movemode = MOVE_ZOOM;
		if (fastwireframe 
			&& (rendermode != DM_FWF && rendermode != DM_FAT)) {
			savemode = rendermode;
			if (rendermode == DM_AT) {
				rendermode = DM_FAT;
			} else {
				rendermode = DM_FWF;
			}
		}
		lpos[V_X] = x;
		lpos[V_Y] = y;
		glutPostRedisplay();
		break;
	case 0:
		movemode = MOVE_NONE;
		if (rendermode == DM_FWF || rendermode == DM_FAT) {
			rendermode = savemode;
		}
		quality = staticquality;
		glutPostRedisplay();
		break;
	}
}

void
mouseMovement(int x, int y)
{
	switch (movemode) {
	case MOVE_TRACKBALL:
		trackball(lastqrot,
			(2.0 * (float)lpos[V_X] - (float)winsize[V_X]) / (float)winsize[V_X],
			((float)winsize[V_Y] - 2.0 * (float)lpos[V_Y]) / (float)winsize[V_Y],
			(2.0 * (float)x - (float)winsize[V_X]) / (float)winsize[V_X],
			((float)winsize[V_Y] - 2.0 * (float)y) / (float)winsize[V_Y]);		
		add_quats(lastqrot, qrot, qrot);
		break;
	case MOVE_ZOOM:
		zoom += 2*(double)(y -lpos[V_Y]) / (double)winsize[V_Y];
		break;
	case MOVE_DEPTHADJUST:
		depthpoint += 4*(double)(y -lpos[V_Y]) / (double)winsize[V_Y];
		break;
	case MOVE_INTEREYEADJUST:
		intereye += (double)(x-lpos[V_X]) / (4 * (double)winsize[V_X]);
		break;
	default:
		break;
	}
	if (movemode != MOVE_NONE) {
		lpos[V_X] = x;
		lpos[V_Y] = y;
		glutPostRedisplay();
	}
}

void
standardKeypress(unsigned char key, int x, int y)
{
	switch (key) {
	case '\033':	/* ESC  -- Quit */
		terminateApp(0);
		break;
	default:
		break;
	}
}

void
specialKeypress(int key, int x, int y)
{
	switch (key) {
	case GLUT_KEY_F1:
		singlecamloc = CAM_CENTRE;
		glutPostRedisplay();
		break;
	case GLUT_KEY_F2:
		singlecamloc = CAM_LEFTEYE;
		glutPostRedisplay();
		break;
	case GLUT_KEY_F3:
		singlecamloc = CAM_RIGHTEYE;
		glutPostRedisplay();
		break;
	default:
		break;
	}
}


int
main (int argc, char **argv)
{
	int		glutflags;
	int		dbuf = 1;
	int		stereo = 0;
	int		fullscreen = 0;
	int		i;
	char		wintitle[MAXSIZE_WINTITLE];

#ifndef WIN32	
	/* locale support */
	setlocale (LC_ALL, "");
	bindtextdomain (PACKAGE, LOCALEDIR);
	textdomain (PACKAGE);
#endif

	/* initialise glut.  This happens first to remove options that'll
	 * break getopt
	*/
	glutInit(&argc, argv);

	/* parse remaining options */
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-fullscreen")) {
			fullscreen = 1;
		} else if (!strcmp(argv[i], "-nodbuf")) {
			dbuf = 0;
		} else if (!strcmp(argv[i], "-stereo")) {
			stereo = 1;
		} else if (!strcmp(argv[i], "-file")) {
			if (xyzname != NULL) {
				free (xyzname);
				xyzname = NULL;
			}
			xyzname = strdup(argv[++i]);
		} else if (!strcmp(argv[i], "-fwf")) {
			fastwireframe = 1;
		} else {
			usage(argv[0]);
			exit (0);
		}
	}


	if (stereo && !dbuf) {
		fprintf (stderr, _("You can NOT disable doublebuffering and use stereo at the same time.\n"));
		exit (1);
	}

	/* set up display mode */
	glutflags = GLUT_RGBA | GLUT_DEPTH;
	if (stereo) {
		glutflags |= GLUT_STEREO;
		oflags |= OF_STEREO;
	}
	if (dbuf) {
		glutflags |= GLUT_DOUBLE;
	}
	glutInitDisplayMode(glutflags);

	/* create the application window */
	mainWin = glutCreateWindow("xfpd");
	glutDisplayFunc(displayFunc);
	glutReshapeFunc(resizeFunc);
	if (fullscreen) {
		glutFullScreen();
	}

	displaymodeMenu = glutCreateMenu(displaymodeMenuFunc);
	glutAddMenuEntry(_("Space Filling"), DM_SF);
	glutAddMenuEntry(_("Ball and Stick"), DM_BS);
	glutAddMenuEntry(_("Wireframe"), DM_WF);
	glutAddMenuEntry(_("Alpha Carbon Trace"), DM_AT);

	hideMenu = glutCreateMenu(hideMenuFunc);
	glutAddMenuEntry(_("Toggle Hydrogen Atoms"), HM_HYDROGEN);
	glutAddMenuEntry(_("Toggle Water Molecules"), HM_WATER);

	/* set up application menus */
	mainMenu = glutCreateMenu(mainMenuFunc);
	glutAddMenuEntry(_("Toggle Spin"), MM_SPIN);
	if (stereo) {
		glutAddMenuEntry(_("Toggle Stereo"), MM_STEREOTOGGLE);
	}
	glutAddMenuEntry(_("Reload Data"), MM_RELOAD);
	glutAddSubMenu(_("Display Mode"), displaymodeMenu);
	glutAddSubMenu(_("Hide Options"), hideMenu);
	glutAddMenuEntry(_("Quit"), MM_QUIT);
	glutAttachMenu(GLUT_RIGHT_BUTTON);

	glutMouseFunc(mouseButton);
	glutMotionFunc(mouseMovement);
	glutSpecialFunc(specialKeypress);
	glutKeyboardFunc(standardKeypress);

	/* initialise window size */
	winsize[V_X] = glutGet(GLUT_WINDOW_WIDTH);
	winsize[V_Y] = glutGet(GLUT_WINDOW_HEIGHT);
	/* initialise the rotation */
	trackball(qrot, 0.0, 0.0, 0.0, 0.0);

	/* load the configuration file */
	loadConfiguration();

	/* prime the system */
	if (readxyz() != FAH_OK) {
		fprintf (stderr, _("Failed to read data\n"));
		exit (1);
	}
	quality = staticquality;

	/* set the title bar */
	snprintf (wintitle, MAXSIZE_WINTITLE, "xfpd: %s", pname);
	wintitle[MAXSIZE_WINTITLE-1] = '\0';
	glutSetWindowTitle(wintitle);

	glutMainLoop();

	/* never reached */
	return 0;
}
