/* xfpd.h
 *
 * Chris' FPD
 *
 * Graphics display of FAH protein for Linux
 *
 * Copyright (C) 2003 Christopher Collins <ccollins@ug.cs.usyd.edu.au>
 * Copyright (C) 2002 Richard P. Howell IV.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *                                                                             
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *                                                                             
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <math.h>
#ifdef WIN32
#  include <windows.h>
#else
#  include <unistd.h>
#  include <locale.h>
#  include <libintl.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#ifdef HAVE_SYSTEM_GLUT
#  include <GL/glut.h>
#else
#  include <glut.h>
#endif
#include "trackball.h"
#include "conffile.h"

#ifndef XFPD_H
#define XFPD_H

#include "config.h"

/* for gettext and other platformisms */
#ifdef WIN32
#  define _(s) (s)
#  define snprintf _snprintf
#else
#  define _(s) gettext(s)
#endif

#define xxstringize(x) #x
#define stringize(x) xxstringize(x)

/* maximum length to use for window titles */
#define MAXSIZE_WINTITLE	80

/* default datafile to load */
#define DEFAULTXYZNAME	"work/current.xyz"
extern char *		defaultdatafile;

/* display menu / rendering mode */
#define	DM_SF		1
#define	DM_BS		2
#define	DM_WF		3
#define	DM_AT		4
/* FWF == Fast WireFrame, FAT == Fast AlphaCarbonTrace */
#define DM_FWF		5
#define DM_FAT		6

/* nullzone for mouse spinning */
#define SPIN_NULLZONE	2

/* global configuration */
extern config_t *	xfpdconf;

/* defined in render.c */
extern int		rendermode;
extern GLfloat		qrot[];
extern double		dv[];
extern double		zoom;
extern int		quality;
extern double		aspect;
extern int		oflags;
extern double		bs_spheresize;
extern double		scale;
extern GLfloat		bgcolor[];

/* main menu */
#define MM_QUIT		1
#define MM_RELOAD	2
#define MM_SPIN		3
#define MM_STEREOTOGGLE 4

/* hide menu */
#define HM_HYDROGEN	1
#define HM_WATER	2

/* vector members */
#define V_X		0
#define V_Y		1
#define V_Z		2

/* output rendering flags */
#define OF_IH		0x0001	/* Don't display hydrogen atoms */
#define OF_IW		0x0002	/* Don't display water molecules */
#define OF_STEREO	0x1000	/* stereo rendering stuff */

/* stereoscopic defaults */
#define INITIAL_INTEREYE	0.012451171875
#define INITIAL_DEPTHPOINT	9.0

/* camera locations */
#define CAM_LEFTEYE	-1
#define CAM_RIGHTEYE	1
#define CAM_CENTRE	0

/* stereo control variables - render.c */
extern int	singlecamloc;
extern float	intereye;
extern float	depthpoint;

/* Atomic radii, bond lengths, and similar things are in Angstroms.
** The display sizes of the atoms are, to some extent, a fudge.
*/
struct atmprop {	/* Atom properties (in ETYPES order) */
	double		radius;		/* Display radius of this element type */
	GLfloat		color[4];	/* Color (R G B A) */
};

extern struct atmprop aprops[];
extern struct atmprop acprops[];

/* functions from render.c */
extern void		renderMolecule();

/* from fah.c */
#define	FAH_OK		0
#define FAH_CANTOPEN	1
#define FAH_TOOLARGE	2
#define FAH_OUTOFMEMORY	3
#define FAH_BADSEQUENCE 4
#define FAH_TOOMANYATOMS 5
#define FAH_TOOMANYBONDS 6
#define FAH_NOATOMS	7

extern char *		xyzname;
extern char 		pname[];


/*** From fpd.c */
#define IZOOM		3.0
#define TIME_OFS	946684800U	/* Unix epoch is 1970, FAH is 2000 */
#define ETYPES		"?HCNOPS"	/* Known elements (in table order) */
#define MAXATOMS	100000		/* Maximum atoms in molecule */
#define MAXBONDS	8		/* Maximum bonds on one atom */

#define rankof(x) (sizeof(x)/sizeof(*x))

struct bnd {
	int		toward;		/* Atom bonded to */
	double		len;		/* Bond length (/2) */
	double		theta;		/* Z-axis rotation */
	double		phi;		/* Y-axis rotation */
};

struct atm {
	short		element;	/* Element */
	short		flags;		/* Flags */
	GLfloat		pos[3];		/* position (coordinates) */
	struct bnd *	achc;		/* Alpha carbon peptide bond toward carbonyl */
	struct bnd *	acha;		/* Alpha carbon peptide bond toward imino */
	int		bondcount;	/* number of bonds */
	struct bnd	bond[MAXBONDS];	/* Bond descriptions */
};

/* Atom flag values */

#define AF_H	0x0001		/* Hydrogen */
#define AF_H2O	0x0002		/* Water */

struct adist {
	float		r;
	struct atm *	pa;
};

/* global stuff -- defined in render.c */
extern int		atoms;	/* Total number of atoms in "atom" table */
extern struct atm *	atom;	/* Pointer to allocated storage for all XYZ file data */

/* fah.h */
extern int		readxyz ();

#endif /* XFPD_H */
